/*
    Project: TCTEMP
    Version: V1.7
    File   : tc_header.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tomcrypt.h"
#include "tc_priv.h"

BOOL __stdcall TC_EncryptHeader(PTC_HEADER pHeader, const void *pPassword,
  size_t cbPasswordSize, unsigned uHash, unsigned uCipher, PTC_BUFFER pBuffer)
{
    BOOL fResult = FALSE;

    if (!TC_HashIsValid(uHash) || !TC_CipherIsValid(uCipher)) {
        return FALSE;
    }

    if (TC_DeriveNonWeakKey(&pBuffer->key, pPassword, cbPasswordSize,
      pHeader->rgbSalt, sizeof(pHeader->rgbSalt), uHash, 0, uCipher,
      &pBuffer->pkcs5_2_state)) {
        if (TCE_Init(&pBuffer->ctx, &pBuffer->key, uCipher)) {

            memset(pHeader->rgbReserved, '\0', sizeof(pHeader->rgbReserved));
            pHeader->dwId = TC_HEADER_ID;
            Reverse32(&pHeader->dwId);
            pHeader->wHeaderVersion = TC_HEADER_VERSION;
            Reverse16(&pHeader->wHeaderVersion);
            pHeader->wReqProgramVersion = TC_REQ_PROGRAM_VERSION;
            Reverse16(&pHeader->wReqProgramVersion);
            pHeader->dwCRC32 = GetCRC32(pHeader->rgbSecondaryKey, 256);
            Reverse32(&pHeader->dwCRC32);

            if (TCE_Encrypt(&pBuffer->ctx, &pHeader->dwId, 512-64)) {
                fResult = TRUE;
            }

            TCE_Close(&pBuffer->ctx);
        }
    }

    return fResult;
}
